import numpy
import math
from shapely import geometry
from shapely.geometry import MultiPolygon, Polygon, mapping, shape
from shapely.geometry.polygon import LinearRing


def fixseries(time_series):
    
    """
    
    This function fix the time series.
    
    As some time series may have very significant noises. When coverted to polar space
    it may produce inconsistent polygons. To avoid this, this function remove this spikes.

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.ndarray:
	   Numpy array of time series without spikes.
    
    """

    check_input(time_series)

    if time_series.size == numpy.ones((1,)).size :
        return numpy.array([1])
    
    time_series2 = time_series
    
    idxs = numpy.where(time_series == 0)[0]
    spikes = list()
    for i in range(len(idxs)-1):
        di = idxs[i+1]-idxs[i]
        if di==2:
            spikes.append(idxs[i]+1)  

    for pos in range(len(spikes)):
        idx = spikes[pos]
        time_series2[idx]=0
    
    if len(time_series2) <= 3:
        raise TypeError("Your time series has too much noise we cant compute metrics!")
    
    return time_series2

def create_polygon(timeseries):
    
    """
    
    This function converts the time series to the polar space.
    If the time series has lenght smaller than 3, it cannot be properly converted
    to polar space.

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
        Shapely.Polygon
    
    """
    
    import math

    #remove weird spikes on timeseries
    ts = fixseries(timeseries)

    if ts.size == numpy.ones((1,)).size :
        return numpy.array([1])
     
    list_of_radius, list_of_angles = get_list_of_points(ts)

    # create polygon geometry
    ring = list()
    
    # add points in the polygon
    N = len(list_of_radius)
    
    #start to build up polygon
    for i in range(N):
        a = list_of_radius[i] * math.cos(2 * numpy.pi * i / N)
        o = list_of_radius[i] * math.sin(2 * numpy.pi * i / N)
        ring.append([a,o])
    r = LinearRing(ring)  
    
    try:
        polygon = Polygon(r)
    except:
        print("Unable to create a valid polygon")
    
    return polygon

def get_list_of_points(ts):
    
    """
    
    This function creates a list of angles based on the time series.
    This list is used for convert the time series to a polygon.

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    lists:
        Lists of observations and angles, that are used to create a polygon.
    
    """
    
    list_of_observations = abs(ts)

    list_of_angles = numpy.linspace(0, 2 * numpy.pi, len(list_of_observations))
    
    return list_of_observations, list_of_angles

def check_input(timeseries):

    """
    
    This function check the inumpyut and raise exception if it is too short or 
    has the wrong type. 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
        timeseries
    
    """
    if timeseries.size == numpy.ones((1,)).size :
        return None

    if isinstance(timeseries,numpy.ndarray):

        if len(timeseries) < 3:
            raise TypeError("Your time series is too short!")
        else:
            return timeseries
    else:
        raise Exception('Incorrect type: Please use numpy.array as inumpyut.')





import numpy
from math import log, floor
import nolds

from.utils import *

def dfa_fd(series):
    """Detrended Fluctuation Analysis (DFA)

    DFA measures the Hurst parameter H, which is very similar to the Hurst exponent. 
    The main difference is that DFA can be used for non-stationary processes (whose mean and/or variance change over time).

    Keyword arguments:
    series : numpy.array
        One dimensional time series.
    Returns
    -------
    dfa : float
        Detrended Fluctuation Analysis.
    
    Notes:
    ------
    This functions uses the dfa implementation from the Nolds package.
    """

    dfa = nolds.dfa(series)
    return dfa

def hurst_exp(series):
    """
    Hurst exponent.
    Hurst Exponent is a self-similarity measure that assess long-range dependence in a time series.
    
    Keyword arguments:
    series : numpy.array
        One dimensional time series.
    Returns
    -------
    hurst : float
        Hurst exponent.
    
    The hurst exponent is a measure of the “long-term memory” of a time series. 
    It can be used to determine whether the time series is more, less, or equally likely to increase if it has increased in previous steps. 
    This property makes the Hurst exponent especially interesting for the analysis of stock data.
    """

    h = nolds.hurst_rs(series)
    return h

def petrosian_fd(series):
    """Petrosian Algorithm.

    This algorirhm computes the FD of a signal by translating the series into a binary sequence.

    Keyword arguments:
    series : numpy.array
        One dimensional time series.
    Returns
    -------
    pfd : float
        Petrosian fractal dimension.
    Notes
    -----
    The Petrosian fractal dimension of a time-series ..:math:`x` is defined by:
    .. math:: P = \\frac{\\log_{10}(N)}{\\log_{10}(N) +
              \\log_{10}(\\frac{N}{N+0.4N_{\\delta}})}
    where ..:math:`N` is the length of the time series, and
    ..:math:`N_{\\delta}` is the number of sign changes in the signal derivative.

    This function was extracted from the package, available at: https://github.com/raphaelvallat/entropy.

    References
    ----------
    .. [1] A. Petrosian, Kolmogorov complexity of finite sequences and
       recognition of different preictal EEG patterns, in , Proceedings of the
       Eighth IEEE Symposium on Computer-Based Medical Systems, 1995,
       pp. 212-217.
    .. [2] Goh, Cindy, et al. "Comparison of fractal dimension algorithms for
       the computation of EEG biomarkers for dementia." 2nd International
       Conference on Computational Intelligence in Medicine and Healthcare
       (CIMED2005). 2005.
    """
    n = len(series)
    # Number of sign changes in the first derivative of the signal
    diff = numpy.ediff1d(series)
    N_delta = (diff[1:-1] * diff[0:-2] < 0).sum()
    return numpy.log10(n) / (numpy.log10(n) + numpy.log10(n / (n + 0.4 * N_delta)))


def katz_fd(series):
    """Katz Algorithm.
    
    Keyword arguments:
    series : numpy.array. 
        One dimensional time series.
        
    Returns
    -------
    kfd : float
        Katz fractal dimension.
    Notes
    -----
    The Katz fractal dimension is defined by:

    .. math:: K = \\frac{\\log_{10}(n)}{\\log_{10}(d/L)+\\log_{10}(n)}
    
    where :math:`L` is the total length of the time series and :math:`d`
    is the
    `Euclidean distance <https://en.wikipedia.org/wiki/Euclidean_distance>`_
    between the first point in the series and the point that provides the
    furthest distance with respect to the first point.

    This function was extracted from the package, available at: https://github.com/raphaelvallat/entropy.

    References
    ----------
    .. [1] Esteller, R. et al. (2001). A comparison of waveform fractal
           dimension algorithms. IEEE Transactions on Circuits and Systems I:
           Fundamental Theory and Applications, 48(2), 177-183.
    .. [2] Goh, Cindy, et al. "Comparison of fractal dimension algorithms for
           the computation of EEG biomarkers for dementia." 2nd International
           Conference on Computational Intelligence in Medicine and Healthcare
           (CIMED2005). 2005.
    """
    x = numpy.array(series)
    dists = numpy.abs(numpy.ediff1d(x))
    ll = dists.sum()
    ln = numpy.log10(numpy.divide(ll, dists.mean()))
    aux_d = x - x[0]
    d = numpy.max(numpy.abs(aux_d[1:]))
    return numpy.divide(ln, numpy.add(ln, numpy.log10(numpy.divide(d, ll))))


# def _higuchi_fd(series, kmax):
#     """Utility function for `higuchi_fd`.

#     Notes
#     -----
#     This function was extracted from the package, available at: https://github.com/raphaelvallat/entropy.

#     """
#     n_times = series.size
#     lk = numpy.empty(kmax)
#     x_reg = numpy.empty(kmax)
#     y_reg = numpy.empty(kmax)
#     for k in range(1, kmax + 1):
#         lm = numpy.empty((k,))
#         for m in range(k):
#             ll = 0
#             n_max = floor((n_times - m - 1) / k)
#             n_max = int(n_max)
#             for j in range(1, n_max):
#                 ll += abs(series[m + j * k] - series[m + (j - 1) * k])
#             ll /= k
#             ll *= (n_times - 1) / (k * n_max)
#             lm[m] = ll
#         # Mean of lm
#         m_lm = 0
#         for m in range(k):
#             m_lm += lm[m]
#         m_lm /= k
#         lk[k - 1] = m_lm
#         x_reg[k - 1] = log(1. / k)
#         y_reg[k - 1] = log(m_lm)
#     higuchi, _ = _linear_regression(x_reg, y_reg)
#     return higuchi


# def higuchi_fd(series, kmax=10):
#     """Higuchi Fractal Dimension (HFD).

#     HFD is defined as the slope of the line that fits the pairs {ln[L(k)],ln(1/k)} in a least-squares sense.  

#     where: k indicates the discrete time interval between points.

#     Keyword arguments:
#     ----------
#     x : list or numpy.array
#         One dimensional time series.
#     kmax : int
#         Time interval between points. 
#     Returns
#     -------
#     hfd : float
#         Higuchi fractal dimension.
#     Notes
#     -----
#     This function was extracted from the package, available at: https://github.com/raphaelvallat/entropy.

#     References
#     ----------
#     .. [1] Higuchi, Tomoyuki. "Approach to an irregular time series on the
#        basis of the fractal theory." Physica D: Nonlinear Phenomena 31.2
#        (1988): 277-283.
#     """
#     x = numpy.asarray(series, dtype=numpy.float64)
#     kmax = int(kmax)
#     return _higuchi_fd(series, kmax)

def ts_fractal(timeseries,kmax=10):
    
    """
    This function compute 4 fractal dimensions and the hurst exponential.
    
    DFA: measures the Hurst parameter H, which is very similar to the Hurst exponent.
    HE: self-similarity measure that assess long-range dependence in a time series.
    KFD:
    PFD: This algorirhm computes the FD of a signal by translating the series into a binary sequence.
    
    Keyword arguments:
    timeseries : numpy.ndarray
        Your time series.
    Returns
    -------
    numpy.array:
        array of fractal metrics values
    """
    
    #Compute metrics

    ts = fixseries(timeseries)
    
    dfa = dfa_fd(ts)
    #hfd = higuchi_fd(ts, kmax=kmax)
    he = hurst_exp(ts)
    kfd = katz_fd(ts)
    pfd = petrosian_fd(ts) 

    return numpy.array([dfa,he,kfd,pfd])

# def _linear_regression(x, y):
#     """Fast linear regression using Numba.
#     Parameters
#     ----------
#     x, y : ndarray, shape (n_times,)
#         Variables
#     Returns
#     -------
#     slope : float
#         Slope of 1D least-square regression.
#     intercept : float
#         Intercept
#     """
#     n_times = x.size
#     sx2 = 0
#     sx = 0
#     sy = 0
#     sxy = 0
#     for j in range(n_times):
#         sx2 += x[j] ** 2
#         sx += x[j]
#         sxy += x[j] * y[j]
#         sy += y[j]
#     den = n_times * sx2 - (sx ** 2)
#     num = n_times * sxy - sx * sy
#     slope = num / den
#     intercept = numpy.mean(y) - slope * numpy.mean(x)
#     return slope, intercept


# def _log_n(min_n, max_n, factor):
#     """
#     Creates a list of values by successively multiplying a minimum value min_n by
#     a factor > 1 until a maximum value max_n is reached.
#     Non-integer results are rounded down.
#     Args:
#       min_n (float):
#         minimum value (must be < max_n)
#     max_n (float):
#         maximum value (must be > min_n)
#     factor (float):
#         factor used to increase min_n (must be > 1)
#     Returns:
#         list of integers:
#             min_n, min_n * factor, min_n * factor^2, ... min_n * factor^i < max_n
#             without duplicates
#     """
#     #assert max_n > min_n
#     #assert factor > 1
#     # stop condition: min * f^x = max
#     # => f^x = max/min
#     # => x = log(max/min) / log(f)
#     max_i = int(numpy.floor(numpy.log(1.0 * max_n / min_n) / numpy.log(factor)))
#     ns = [min_n]
#     for i in range(max_i + 1):
#         n = int(numpy.floor(min_n * (factor ** i)))
#         if n > ns[-1]:
#             ns.append(n)

#     return ns


import math
import numpy
import pandas
import matplotlib.pyplot as plt
from sklearn.metrics import auc
from scipy.signal import savgol_filter,medfilt,find_peaks


def get_filtered_series(timeseries,window,treshold):
    
    """
    
    This function filter the input timeseries using the Savitzky-Golay method, using \\
    a similar approach to the Timesat. 
    
    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.
        treshold : float
            Minimum growing that will be used to detect a new cycle.
        window: integer (odd number) default 7
            Size of the window used for filtering with the Savitzky-Golay 
    Returns
    -------
    numpy.ndarray:
        filtered timeseries (sg)
    
    """
    
    min_win = int(numpy.floor(window/2))
    y = numpy.empty([min_win, timeseries.shape[0]])
    
    pos = 0
    for i in range(window,3,-2):
        y[pos,:] = savgol_filter(timeseries,i,2)
        pos+=1

    diff = abs(y[0]/timeseries)
    diff[numpy.isnan(diff)]=0
    diff[numpy.isinf(diff)]=0
    
    max_diff = abs(numpy.max(diff))

    level = 0
    sg = y[0]

    while max_diff > treshold and level < pos-1:
        new = y[level+1,:]
        sg = numpy.where(diff>treshold,new,sg)
        diff = abs(sg/timeseries)
        diff[numpy.isnan(diff)]=0
        diff[numpy.isinf(diff)]=0
        max_diff = abs(numpy.max(diff))
        level += 1
    
    return sg

def get_ts_metrics(y,a,c,l_mi,b,d,r_mi,tresh):

    """
    
    This function compute the phenological metrics
    
    Reference: 

    Keyword arguments:
       
    Returns
    -------
    numpy.ndarray:
        array of peaks
    """

    #Interpolate a, b, c and d positions
    #Fine adjustment
    xp = numpy.arange(0,len(y))
    start_val = numpy.interp(a, xp, y)
    end_val = numpy.interp(b, xp, y)
    yc = numpy.interp(c, xp, y)
    yd = numpy.interp(d, xp, y)
    
    #Compute other metrics
    l_derivada = abs(start_val-yc)/abs(a-c)
    r_derivada = abs(end_val-yd)/abs(b-d)  
    lenght = abs(a-b)
    Base_val = (l_mi-r_mi)/2
    Peak_val = numpy.max(y[int(a):int(b)])
    Peak_t = list(y[int(a):int(b)]).index(Peak_val)+a
    ampl = Peak_val - Base_val

    #compute areas
    xx = numpy.arange(int(a),int(b),1)
    yy = y[int(a):int(b)]
    yy2 = abs(y[int(a):int(b)])+abs(Base_val)
    h = auc(xx,yy)
    i = auc(xx,yy2)
    
    return numpy.array([a,b,lenght,Base_val,Peak_t,Peak_val,ampl,l_derivada,r_derivada,h,i,start_val,end_val])

def get_greenness(series,start,minimum_up):


    """
    
    This function get info about the browness of a cycle.
    
    Reference: 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.
        start : integer 
            Position along time series axis
        minimum_up : float 
            Minimum growing that will be used to detect a new cycle.

    Returns
    -------
    numpy.ndarray:
        array of peaks
    """
    
    #funtion to get the start/end of interval
    idx = numpy.where(series == numpy.amin(series))[0][0]
    series = series[idx:]
    l_mi = min(series)
    ma = max(series)
    dis = ma-l_mi

    #get cummulative sum from left side
    ret = ((series - l_mi)/ma)
    xp = numpy.arange(0,len(ret))    
    c = numpy.interp(1-minimum_up, ret, xp) + start 
    a = numpy.interp(minimum_up, ret, xp) + start
    
    return a,c,l_mi

def get_brownness(series,start,minimum_up):

    """
    
    This function get info about the browness of a cycle.
    
    Reference: 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.
        start : integer 
            Position along time series axis
        minimum_up : integer 
            Minimum growing that will be used to detect a new cycle.

    Returns
    -------
    numpy.ndarray:
        array of peaks
    """
    
    #funtion to get the start/end of interval  
    idx = numpy.where(series == numpy.amin(series))[0][0]
    
    series = series[:idx]

    r_mi = min(series)
    ma = max(series)
    dis = ma-r_mi

    #get cummulative sum from left side
    series = series[::-1]
    
    ret = (series - r_mi)/ma
    xp = numpy.arange(0,len(ret))    

    d = start + idx - numpy.interp(1-minimum_up, ret, xp) + 1
    b = start + idx - numpy.interp(minimum_up, ret, xp) + 1
    
    return b,d,r_mi

def get_peaks(timeseries,minimum_up):
    
    """
    
    This function find the peaks of a time series.
    
    Reference: 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.
        minimum_up : float
            Minimum growing that will be used to detect a new cycle.
    Returns
    -------
    numpy.ndarray:
        array of peaks
    """
    

    b = (numpy.diff(numpy.sign(numpy.diff(timeseries))) > 0).nonzero()[0] +1# local min
    c = (numpy.diff(numpy.sign(numpy.diff(timeseries))) < 0).nonzero()[0] +1# local max

    picos = []
    for vale, pic in zip(b,c):
        
        if abs(timeseries[vale] - timeseries[pic]) <= minimum_up:
            continue
        else:
            picos.append(vale)
            picos.append(pic)

    return numpy.sort(numpy.asarray(picos))


def domain(y,peaks,minimum_up):

    """
    
    This function find the cycles inside a timeseries.
    
    Reference: 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.
        peaks : list
            Peaks that are detected using function get_peaks.
        minimum_up : float
            Minimum growing that will be used to detect a new cycle.

    Returns
    -------
    pandas.dataframe:
        dataframe with the phenometrics
    """
    
    
    header_timesat=["Start","End","Length","Base val.","Peak t.","Peak val.","Ampl.","L. Deriv.","R. Deriv.","L.integral","S.integral","Start val.","End val."]
    dfp = pandas.DataFrame(columns=header_timesat)
    
    pos = 0
    
    for peak in range(1,len(peaks)-1):
        
        start = peaks[peak-1]
        midle = peaks[peak]
        end = peaks[peak+1]
        
        if y[midle] < y[start] or y[midle] < y[end]:
            continue
            
        elif (y[midle] - y[start]) <= minimum_up:
            continue    
           
        elif (y[midle] - y[end]) <= minimum_up :
            
            cond = (y[midle] - y[end]) >= minimum_up
            pp=peak+1
            
            while cond != True and pp <= peaks.shape[0]-1:
                end = peaks[pp]
                cond = (y[midle] - y[end]) > minimum_up
                pp = pp+1

        series = y[start:midle]    
         
        if series.shape[0] <= 2:
            continue
                
        #get left side
        a,c,l_mi = get_greenness(series,start,minimum_up)

        #get right side
        series = None
        series = y[midle:end]
                
        if series.shape[0] <= 2:
            continue
                
        b,d,r_mi = get_brownness(series,midle,minimum_up)
    
        dfp.loc[pos] = get_ts_metrics(y,a,c,l_mi,b,d,r_mi,minimum_up)
        
        dfp = dfp.dropna()        

        pos +=1
                    
    return dfp

def metrics(time_series, minimum_up, treshold = 0.125, window = 7, show=True):
    
    """
    
    This function get the phenological metrics.
    
    Reference: 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.
        minimum_up : float
            Minimum growing that will be used to detect a new cycle.
        treshold: float, 0.125 default
            Maximum distance used
        Window: integer (odd number) default 7
            Size of the window used for filtering with the Savitzky-Golay 
        show: boolean
            This inform if you want to plot the series with the starting and ending of each cycle detected.
    Returns
    -------
    pandas.dataframe:
        dataframe with the phenometrics
    timeseries plot with the start and end points
    """
    
    #This function perform the filtering with the savitky-golay mehtod
    y = get_filtered_series(time_series,window,max_dist_fitting)
    
    #This function detect peaks on the timesries
    peaks = get_peaks(y,window,minimum_up) 
    
    #This functions detect cycles and compute phenometrics from each of them
    dfp = domain(y,peaks,minimum_up)
    
    #This show the plot with the timeseries
    if show == True:
        fig, ax = plt.subplots(figsize=(15, 5))
        plt.plot(time_series,label='TimeSeries')
        plt.plot(y,label='SG-Phenometrics')
        ax.scatter(dfp['Start'],dfp['Start val.'],label='Start point',color='green')
        ax.scatter(dfp['End'],dfp['End val.'],label='End point',color='red')
        legend = ax.legend(loc='uper left', shadow=True, fontsize='large',ncol=5,bbox_to_anchor=(1,1.15))
        plt.show()
    
    return dfp


import numpy
import matplotlib.pyplot as plt
from descartes import PolygonPatch
from shapely import geometry
from shapely.geometry import MultiPolygon, Polygon, mapping, shape
from shapely.geometry.polygon import LinearRing

from .utils import *

def symmetric_distance(time_series_1, time_series_2):

    """
    
    This function computes the difference between two time series considering the
    polar space.

    Keyword arguments:
        time_series_1 : numpy.ndarray
        time_series_2 : numpy.ndarray    

    Returns
    -------
    numpy.float64:
        distance
    
    """
    import math

    dist = math.inf
    pos = math.inf


    time_series_1 = fixseries(time_series_1)
    time_series_2 = fixseries(time_series_2)

    polygon_1 = create_polygon(time_series_1)
    
    if polygon_1.is_valid == False:
        polygon_1 = polygon_1.buffer(0)
    
    if min(time_series_1) > max(time_series_2) or min(time_series_2) > max(time_series_1):
        polygon_2 = create_polygon(time_series_2)
        polygons_symmetric_difference = polygon_1.symmetric_difference(polygon_2)
        dist = polygons_symmetric_difference.area
        
    else:
        for i in range(len(time_series_1)):
            shifted_time_series_2 = numpy.roll(time_series_2, i)
            temp = numpy.linalg.norm(time_series_1-shifted_time_series_2)

            if temp<dist:
                dist = temp
                pos = i
               
        time_series_2 = numpy.roll(time_series_2,pos)

        polygon_2 = create_polygon(time_series_2)

        if polygon_2.is_valid:

            polygons_symmetric_difference = polygon_1.symmetric_difference(polygon_2)

            dist = polygons_symmetric_difference.area

        else:

            polygon_2 = polygon_2.buffer(0)

            polygons_symmetric_difference = polygon_1.symmetric_difference(polygon_2)

            dist = polygons_symmetric_difference.area
        
    return dist


def polar_plot(timeseries):
    
    """
    
    This function create a plot of time series in polar space.

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    None
    Plot of time series in polar space.
    
    """
    ts = fixseries(timeseries)
    polygon = create_polygon(ts)
    
    x,y = polygon.envelope.exterior.coords.xy
    minX = -numpy.max(numpy.abs(x))
    minY = -numpy.max(numpy.abs(y))
    maxX = numpy.max(numpy.abs(x))
    maxY = numpy.max(numpy.abs(y))
    
    ringTopLeft,ringTopRight,ringBottomLeft,ringBottomRight = get_seasons(x,y)
    
    fig = plt.figure()
    ax = fig.add_subplot(111)
    patch = PolygonPatch(ringTopLeft,fc='#F7CF89', ec=None, alpha=0.5)
    patch2 = PolygonPatch(ringTopRight,fc='#8BF789', ec=None, alpha=0.5)
    patch3 = PolygonPatch(ringBottomLeft,fc='#D09A3C', ec=None, alpha=0.5)
    patch4 = PolygonPatch(ringBottomRight,fc='#6CBEEA', ec=None, alpha=0.5)
    patch5 = PolygonPatch(polygon,fc='#EF8C78', ec=None, alpha=0.5)
    plt.ylim(top=maxY)  # adjust the top
    plt.ylim(bottom=minY)  # adjust the bottom 
    plt.xlim(left=minX)  # adjust the left
    plt.xlim(right=maxX)  # adjust the right
    ax.add_patch(patch)
    ax.add_patch(patch2)
    ax.add_patch(patch3)
    ax.add_patch(patch4)
    ax.add_patch(patch5)
    plt.show()
    
    return None


def get_seasons(x,y):
    
    """
    
    This function polygons that represents the four season of a year.
    They are used to compute the metric "area_season."
    
    Keyword arguments:
        x,y coordinates:
		Using this coordinates is built the quadrants that represents seasons.

    Returns
    -------
	Polygons
    
    """
    
    minX = -numpy.max(numpy.abs(x))
    minY = -numpy.max(numpy.abs(y))
    maxX = numpy.max(numpy.abs(x))
    maxY = numpy.max(numpy.abs(y))

    '''
    coord0----coord1----coord2
    |           |           |
    coord3----coord4----coord5
    |           |           |
    coord6----coord7----coord8
    '''

    coord0 = minX, maxY
    coord1 = 0, maxY
    coord2 = maxX, maxY
    coord3 = minX, 0
    coord4 = 0,0
    coord5 = maxX, 0
    coord6 = minX, minY
    coord7 = 0, minY
    coord8 = maxX, minY

    polyTopLeft = Polygon([coord0,coord3,coord4,coord1,coord0])
    polyTopRight = Polygon([coord1,coord2,coord5,coord4,coord1])
    polyBottomLeft = Polygon([coord3,coord4,coord7,coord6,coord3])
    polyBottomRight = Polygon([coord4,coord5,coord8,coord7,coord4])
    
    return polyTopLeft,polyTopRight,polyBottomLeft,polyBottomRight

def area_season(timeseries):
    
    """

    Area per season - Partial area of the shape, proportional to some quadrant of the polar representation.
    This metric returns the area of the polygon on each quadrant.
    
    Reference: Körting, Thales & Câmara, Gilberto & Fonseca, Leila. (2013). \\
    Land Cover Detection Using Temporal Features Based On Polar Representation. 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64
    	The area of the time series that intersected each quadrant that represents a season.
    
    """
    ts = fixseries(timeseries)

    #create polygon
    polygon = create_polygon(ts) 
    
    """
    area2----area1
    |           | 
    area3----area4
    """
    
    x,y = polygon.envelope.exterior.coords.xy
    
    polyTopLeft,polyTopRight,polyBottomLeft,polyBottomRight = get_seasons(x,y)
    
    quaterPolyTopLeft = polyTopLeft.intersection(polygon)
    quaterPolyTopRight =  polyTopRight.intersection(polygon)
    quaterPolyBottomLeft =  polyBottomLeft.intersection(polygon)
    quaterPolyBottomRight =  polyBottomRight.intersection(polygon)
    
    area1 =  quaterPolyTopRight.area
    area2 = quaterPolyTopLeft.area
    area3 =  quaterPolyBottomLeft.area
    area4 =  quaterPolyBottomRight.area
    
    return area1,area2,area3,area4

def ecc_metric(timeseries):
    
    """

    Eccenticity - Return values close to 0 if the shape is a circle and 1 if the shape is similar to a line.    
    
    Reference: Körting, Thales & Câmara, Gilberto & Fonseca, Leila. (2013). \\
    Land Cover Detection Using Temporal Features Based On Polar Representation. 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64
	Eccentricity of time series.
    
    """


    ts = fixseries(timeseries)
    #create polygon
    polygon = create_polygon(ts)     
    rrec = polygon.minimum_rotated_rectangle
    minx, miny, maxx, maxy = rrec.bounds
    axis1 = maxx - minx
    axis2 = maxy - miny
    stats = numpy.array([axis1, axis2])
    ecc = (stats.min() / stats.max())
    
    return ecc

def angle(timeseries):
    
    """
    Angle - The main angle of the closed shape created by the polar visualization.
    If two angle are the same, the first one is presented
    
    Reference: Körting, Thales & Câmara, Gilberto & Fonseca, Leila. (2013). \\
    Land Cover Detection Using Temporal Features Based On Polar Representation. 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64:
    	The main angle of time series.
    
    """
    ts = fixseries(timeseries)

    list_of_radius, list_of_angles = get_list_of_points(ts)
    index = numpy.argmax(list_of_radius)
    angle = list_of_angles[index]
    
    return angle

def gyration_radius(timeseries):
    
    """
    Gyration_radius - Equals the average distance between each point inside the shape and the shape’s centroid.
    
    Reference: Körting, Thales & Câmara, Gilberto & Fonseca, Leila. (2013). \\
    Land Cover Detection Using Temporal Features Based On Polar Representation. 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64
	Average distance between each point inside the shape and the shape’s centroid.
    
    """
    
    ts = fixseries(timeseries)
    #create polygon
    polygon = create_polygon(ts)   
    
    lonc,latc = polygon.centroid.xy
 
    x,y = polygon.exterior.coords.xy

    dist = []

    for p in range(len(x)):
        px = x[p]
        py = y[p]

        dist = numpy.sqrt((px-lonc[0])**2 + (py-latc[0])**2)

    gyro = numpy.mean(dist)      
    
    return gyro

def polar_balance(timeseries):
    
    """
    Polar_balance - The standard deviation of the areas per season, considering the 4 seasons. 
    
    Reference: Körting, Thales & Câmara, Gilberto & Fonseca, Leila. (2013). \\
    Land Cover Detection Using Temporal Features Based On Polar Representation. 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.
    
    Returns
    -------
    numpy.float64
	Standard deviation of the areas per season.
    
    """

    ts = fixseries(timeseries)

    areas = area_season(ts)
    
    balance = numpy.std(areas)
    
    return balance

def area_ts(timeseries):
    
    """
    Area - Area of the closed shape.
    
    Reference: Körting, Thales & Câmara, Gilberto & Fonseca, Leila. (2013). \\
    Land Cover Detection Using Temporal Features Based On Polar Representation. 
    

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64
	Area of polygon.
    
    """

    ts = fixseries(timeseries)

    #create polygon
    polygon = create_polygon(ts)   
    
    return polygon.area

def ts_polar(timeseries,show = False):
    
    """
    
    This function compute 9 polar metrics:
    
    Area - Area of the closed shape.
    Area_q1 - "Area_q4" - Partial area of the shape, proportional to some quadrant of the polar representation
    Circle - Return values close to 0 if the shape is a circle and 1 if the shape is similar to a line.
    Gyration_radius - Equals the average distance between each point inside the shape and the shape’s centroid.
    Polar_balance - The standard deviation of the areas per season, considering the 4 seasons. 
    Angle - The main angle of the closed shape created by the polar visualization.
    
    To visualize the time series on polar space use: ts_polar(ts,show=True)
    
    Reference: Körting, Thales & Câmara, Gilberto & Fonseca, Leila. (2013). \\
    Land Cover Detection Using Temporal Features Based On Polar Representation. 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.
	show: boolean
	     This inform that the polar plot must be presented.
    Returns
    -------
    numpy.array:
        array of polar metrics values
    polar plot
    """
    
    #define header for polar dataframe
    #header_polar=["Area", "Area_q1", "Area_q2", "Area_q3", "Area_q4","Circle","Gyration_radius","Polar_balance"]
    
    #Compute metrics

    ts = fixseries(timeseries)
    
    #Eccentricity    
    circle = ecc_metric(ts)
    
    #gyration_radius
    gyro = gyration_radius(ts)
    
    #Get Area
    area = area_ts(ts)
    
    #Seasonal area
    areas1,areas2,areas3,areas4 = area_season(ts)  
    
    #Polar Balance
    balance = polar_balance(ts)
    
    #Angle    
    ang = angle(ts)
    
    if show==True:
        polar_plot(ts)
    
    return numpy.array([area,areas1,areas2,areas3,areas4,circle,gyro,balance,ang])


import numpy
from . import basics
from . import polar
from . import fractal

def get_metrics(series,show=False):
    
    #Remove eventual nans from timeseries
    ts = series[~numpy.isnan(series)]
    
    if (not numpy.any(ts)) == True:
        return numpy.zeros((1,20))
        
    #call functions
    basicas = basics.ts_basics(ts)
    polares = polar.ts_polar(ts,show)
    fd = fractal.ts_fractal(ts)

    return numpy.concatenate((basicas, polares,fd), axis=None)

def extractMetrics(series):
    import multiprocessing as mp
    
    #Initialize pool
    pool = mp.Pool(mp.cpu_count())
    
    #use pool to compute metrics for each pixel
    #return a list of arrays
    metrics = pool.map(metrics.get_metrics,[serie for serie in series])
    
    #close pool
    pool.close()    
    
    #Conver list to numpy array
    X_m = numpy.vstack(metrics)
    
    #Concatenate time series and metrics
    X_all = numpy.concatenate((series,X_m), axis=1)
    
    return X_all


import numpy

from . import basics
from . import polar
from . import utils
from . import phenometrics
from . import metrics
from . import fractal

# Version of the package

__version__ = "0.1.4"

name = "stmetrics"


import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="stmetrics",
    version="0.1.4",
    author="Anderson Soares, Thales Körting",
    author_email="andersonreis.geo@gmail.com",
    description="A package to compute metrics from Satellite Image Time Series (SITS).",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/andersonreisoares/stmetrics/",
    packages=['stmetrics'],
    install_requires=[
    'scipy',
    'sklearn',
    'pandas',
    'numpy',
    'matplotlib',
    'shapely',
    'descartes',
    'nolds'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 2 - Pre-Alpha"
    ],
) 


import numpy

from.utils import *

def ts_basics(timeseries):
    
    """
    
    This function compute 7 basic metrics
    "Mean" - Average value of the curve along one cycle.
    "Max" - Maximum value of the cycle.
    "Min" - Minimum value of the curve along one cycle.
    "Std" - Standard deviation of the cycle’s values. 
    "Sum" - Sum of values over a cycle. Usually is an indicator of the annual production of vegetation.
    "Amplitude" - The difference between the cycle’s maximum and minimum values.
    "First_slope" - Maximum value of the first slope of the cycle.
    
    Reference: Körting, Thales & Câmara, Gilberto & Fonseca, Leila. (2013). \\
    Land Cover Detection Using Temporal Features Based On Polar Representation. 


    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.array:
        array of basic metrics values

    
    """
    
    #Define header for basic metrics
    #header_basics=["Mean", "Max", "Min", "Std", "Sum","Amplitude","First_slope","Angle"]    
    
    # compute mean, maximum, minimum, standart deviation and amplitude    
    ts = fixseries(timeseries)

    if ts.size == numpy.ones((1,)).size :
        return numpy.array([1,1,1,1,1,1,1])


    mean = mean_ts(ts)
    maxi = max_ts(ts)
    mini = min_ts(ts)
    std = std_ts(ts)
    soma = sum_ts(ts)
    amp = amplitude(ts)
    slope = first_slop(ts)
    
    return numpy.array([mean,maxi,mini,std,soma,amp,slope])


def mean_ts(timeseries):

    """
    "Mean" - Average value of the curve along one cycle.

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64:
	Mean value of time series.

    """
    ts = fixseries(timeseries)

    if ts.size == numpy.ones((1,)).size :
        return numpy.array([1])

    return numpy.mean(ts)
    
def max_ts(timeseries):

    """
    "Max" - Maximum value of the cycle.

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64:
    	Maximum value of time series.
    """
    ts = fixseries(timeseries)

    if ts.size == numpy.ones((1,)).size :
        return numpy.array([1])

    return numpy.max(ts)

def min_ts(timeseries):

    """
    "Min" - Minimum value of the curve along one cycle.

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64:
    	Minimum value of time series.
    """
    ts = fixseries(timeseries)

    if ts.size == numpy.ones((1,)).size :
        return numpy.array([1])

    return numpy.min(ts)

def std_ts(timeseries):
    """
    "Std" - Standard deviation of the cycle’s values. 

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64:
    	Standard deviation of time series.
    """
    ts = fixseries(timeseries)

    if ts.size == numpy.ones((1,)).size :
        return numpy.array([1])

    return numpy.std(ts)

def sum_ts(timeseries):

    """
    "Sum" - Sum of values over a cycle. 
    Usually is an indicator of the annual production of vegetation.

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64:
    	Sum of values of time series.
    """
    ts = fixseries(timeseries)

    if ts.size == numpy.ones((1,)).size :
        return numpy.array([1])

    return numpy.sum(ts)
    
def amplitude(timeseries):

    """
    "Amplitude" - The difference between the cycle’s maximum and minimum values.

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64:
    	Amplitude of values of time series.
    """
    ts = fixseries(timeseries)

    if ts.size == numpy.ones((1,)).size :
        return numpy.array([1])

    return numpy.max(ts) - numpy.min(ts)
    
def first_slop(timeseries):

    """
    "First_slope" - Maximum value of the first slope of the cycle.
    It indicates when the cycle presents some abrupt change in the curve.

    Keyword arguments:
        timeseries : numpy.ndarray
            Your time series.

    Returns
    -------
    numpy.float64:
    	The maximum value of the first slope of time series.
    """
    ts = fixseries(timeseries)
    
    if ts.size == numpy.ones((1,)).size :
        return numpy.array([1])

    return numpy.max(abs(numpy.diff(ts)))

    